import require$$0 from 'stream';
import require$$0$1 from 'parallel-transform';
import require$$1 from 'cyclist';
import require$$2 from 'util';

var oleoducBareRepoN3a88 = {};

function decorateWithPromise$1(stream, promise) {
  const descriptors = ["then", "catch", "finally"].map((property) => {
    return [property, Reflect.getOwnPropertyDescriptor(Promise.prototype, property)];
  });

  for (const [property, descriptor] of descriptors) {
    const value = (...args) => Reflect.apply(descriptor.value, promise, args);
    Reflect.defineProperty(stream, property, { ...descriptor, value });
  }
  return stream;
}

var decorateWithPromise_1 = { decorateWithPromise: decorateWithPromise$1 };

const { Readable } = require$$0;

/**
 * Adapted from
 *  - https://iximiuz.com/en/posts/nodejs-readable-streams-distilled/
 *  - https://www.derpturkey.com/nodejs-async-generators-for-streaming/
 * @param stream
 * @param options
 * @returns {AsyncGenerator<*, void, *>}
 */
async function* toAsyncIterator$1(stream, options = {}) {
  const chunkSize = options.chunkSize || 1;
  if (typeof stream.read !== "function") {
    stream = Readable.wrap(stream, { objectMode: true });
  }

  let ended = false;
  const onEnded = new Promise((res, rej) => {
    stream.once("error", rej);
    stream.once("end", () => {
      ended = true;
      res();
    });
  });

  while (!ended) {
    const chunk = stream.read(chunkSize);
    if (chunk !== null) {
      yield chunk;
      continue;
    }

    const onReadable = new Promise((res) => stream.once("readable", res));
    await Promise.race([onEnded, onReadable]);
  }
}

var toAsyncIterator_1 = { toAsyncIterator: toAsyncIterator$1 };

const { toAsyncIterator } = toAsyncIterator_1;

function decorateWithAsyncIterator$2(stream) {
  stream[Symbol.asyncIterator] = () => toAsyncIterator(stream);
}

var decorateWithAsyncIterator_1 = { decorateWithAsyncIterator: decorateWithAsyncIterator$2 };

function parseArgs$4(args, defaults = {}) {
  const last = args[args.length - 1];
  const hasOption = typeof last === "object" && !Array.isArray(last) && typeof last.pipe !== "function";
  const options = {
    objectMode: true,
    ...defaults,
    ...(hasOption ? args.pop() : {}),
  };

  return {
    streams: Array.isArray(args[0]) ? args[0] : args,
    options,
  };
}

var parseArgs_1 = { parseArgs: parseArgs$4 };

const { Duplex } = require$$0;

class Duplexer$1 extends Duplex {
  constructor(input, output, options) {
    super(options || { writableObjectMode: true, readableObjectMode: true });
    this.input = input;
    this.output = output;
    this.output.on("end", () => this.push(null));
    this.output.on("data", (chunk) => {
      const backpressure = !this.push(chunk);
      if (backpressure) {
        this.output.pause();
      }
    });
  }
  _write(chunk, encoding, callback) {
    this.input.write(chunk, encoding, callback);
  }
  _final(callback) {
    this.input.end(null, null, callback);
  }
  _read() {
    this.output.resume();
  }
}

var Duplexer_1 = { Duplexer: Duplexer$1 };

const { PassThrough: PassThrough$2 } = require$$0;
const { decorateWithPromise } = decorateWithPromise_1;
const { decorateWithAsyncIterator: decorateWithAsyncIterator$1 } = decorateWithAsyncIterator_1;
const { parseArgs: parseArgs$3 } = parseArgs_1;
const { Duplexer } = Duplexer_1;

function pipeStreamsTogether(streams, wrapper) {
  for (const [i, stream] of streams.entries()) {
    const next = streams[i + 1];
    if (next) {
      stream.pipe(next);
    }

    if (stream !== wrapper) {
      stream.on("error", (err) => wrapper.emit("error", err));
    }
  }
}

function getWrapper(first, last, options = {}) {
  let wrapper;
  if (first === last) {
    wrapper = first;
  } else if (first.writable && last.readable) {
    wrapper = new Duplexer(first, last, options);
  } else if (first.writable) {
    wrapper = first;
  } else if (last.readable) {
    wrapper = last;
  } else {
    wrapper = new PassThrough$2(options);
  }

  return wrapper;
}

function oleoduc$2(...args) {
  const {
    streams,
    options: { promisify, ...rest },
  } = parseArgs$3(args, { promisify: true });

  if (streams.length === 0) {
    throw new Error("You must provide at least one stream");
  }

  const first = streams[0];
  const last = streams[streams.length - 1];
  const wrapper = getWrapper(first, last, rest);

  if (last.readable) {
    decorateWithAsyncIterator$1(wrapper);
  }

  if (promisify) {
    decorateWithPromise(
      wrapper,
      new Promise((resolve, reject) => {
        wrapper.on("error", (e) => reject(e));
        last.on("finish", resolve); // Needed by nodejs 12 and previous
        last.on("close", resolve);
      })
    );
  } else if (wrapper !== last && wrapper !== first) {
    last.on("finish", () => wrapper.emit("finish")); // Needed by nodejs 12 and previous
    last.on("close", () => wrapper.emit("close"));
  }

  pipeStreamsTogether(streams, wrapper);

  return wrapper;
}

var oleoduc_1 = { oleoduc: oleoduc$2 };

const { oleoduc: oleoduc$1 } = oleoduc_1;
const { parseArgs: parseArgs$2 } = parseArgs_1;

function compose$3(...args) {
  const { streams, options } = parseArgs$2(args, { promisify: false });
  return oleoduc$1(streams, options);
}

var compose_1 = { compose: compose$3 };

const transform = require$$0$1;

function transformData$2(handleChunk, options = {}) {
  const { filter, parallel, ...rest } = options;
  const filterChunk = (value) => !filter || filter(value);
  const maxParallel = parallel || 1;

  return transform(maxParallel, { objectMode: true, ...rest }, async (chunk, callback) => {
    try {
      if (!(await filterChunk(chunk))) {
        return callback(null, null);
      }
      const res = await handleChunk(chunk);
      callback(null, res);
    } catch (e) {
      callback(e);
    }
  });
}

var transformData_1 = { transformData: transformData$2 };

const { Transform: Transform$4 } = require$$0;

function transformIntoJSON$1(options = {}) {
  let chunksSent = 0;
  return new Transform$4({
    readableObjectMode: false,
    writableObjectMode: true,
    transform: function (chunk, encoding, callback) {
      const shouldWrap = options.arrayWrapper || options.arrayPropertyName;
      if (chunksSent === 0) {
        if (shouldWrap) {
          let value = JSON.stringify(options.arrayWrapper || {});
          value = value.substring(0, value.length - 1);
          const comma = options.arrayWrapper ? "," : "";
          value += String(`${comma}"${options.arrayPropertyName}":[`);
          this.push(value);
        } else {
          this.push("[");
        }
      }

      if (chunksSent++ > 0) {
        this.push(",");
      }

      this.push(JSON.stringify(chunk));
      callback();
    },
    flush: function (callback) {
      const shouldWrap = options.arrayWrapper || options.arrayPropertyName;

      if (chunksSent === 0) {
        //nothing sent
        if (shouldWrap) {
          const value = options.arrayWrapper || {};
          value[options.arrayPropertyName] = [];
          this.push(JSON.stringify(value));
        } else {
          this.push("[]");
        }
      } else {
        //Close json properly
        this.push("]");
        if (shouldWrap) {
          this.push("}");
        }
      }
      return callback();
    },
  });
}

var transformIntoJSON_1 = { transformIntoJSON: transformIntoJSON$1 };

const { Transform: Transform$3 } = require$$0;

function transformIntoCSV$1(options = {}) {
  const bom = options.bom;
  const separator = options.separator || ";";
  const mapper = options.mapper || ((v) => v);

  function generateColumnNames(chunk, columns) {
    return Object.keys(columns || chunk)
      .map(mapper)
      .join(separator);
  }

  async function generateColumnValues(chunk, columns) {
    let values;
    if (!columns) {
      values = Object.values(chunk);
    } else {
      const columnNames = Object.keys(columns);
      values = await Promise.all(columnNames.map((name) => columns[name](chunk)));
    }

    return values.map(mapper).join(separator);
  }

  let lines = 0;
  return new Transform$3({
    objectMode: true,
    transform: async function (chunk, encoding, callback) {
      const columns = options.columns;

      try {
        if (lines++ === 0) {
          if (bom) {
            this.push("\ufeff");
          }
          this.push(`${generateColumnNames(chunk, columns)}\n`);
        }

        this.push(`${await generateColumnValues(chunk, columns)}\n`);
        callback();
      } catch (e) {
        callback(e);
      }
    },
  });
}

var transformIntoCSV_1 = { transformIntoCSV: transformIntoCSV$1 };

const { Transform: Transform$2 } = require$$0;

class TransformStream extends Transform$2 {
  constructor(handleChunk, options) {
    super(options);
    this.handleChunk = handleChunk;
    this._current = null;
  }

  _transform(chunk, encoding, callback) {
    const res = Promise.resolve(this.handleChunk(chunk));
    const promise = res.then ? res : Promise.resolve(res); //FIXME

    promise
      .then((stream) => {
        stream.on("error", (err) => this.emit("error", err));
        stream.on("data", (data) => {
          this._current = stream;
          const backpressure = !this.push(data);
          if (backpressure) {
            console.log("pause");
            stream.pause();
          }
        });
        stream.on("end", () => {
          this._current = null;
          callback();
        });
      })
      .catch(callback);
  }

  async _read(size) {
    if (this._current) {
      //Ensure every chunks has been consumed from the previous transform
      this._current.resume();
    }
    super._read(size);
  }
}

function transformIntoStream$1(handleChunk, options = {}) {
  return new TransformStream(handleChunk, { objectMode: true, ...options });
}

var transformIntoStream_1 = { transformIntoStream: transformIntoStream$1 };

const { transformData: transformData$1 } = transformData_1;

function filterData$1(filter, options = {}) {
  return transformData$1((data) => data, { ...options, filter });
}

var filterData_1 = { filterData: filterData$1 };

const { Transform: Transform$1 } = require$$0;

function accumulateData$3(accumulate, options = {}) {
  const { accumulator, ...rest } = options;
  let acc = accumulator === undefined ? null : accumulator;
  let flushed = false;

  return new Transform$1({
    objectMode: true,
    ...rest,
    transform: async function (chunk, encoding, callback) {
      try {
        flushed = false;
        const flush = (res) => {
          flushed = true;
          this.push(res);
        };

        acc = await accumulate(acc, chunk, flush);

        callback();
      } catch (e) {
        callback(e);
      }
    },
    flush(callback) {
      if (!flushed) {
        this.push(acc);
      }
      callback();
    },
  });
}

var accumulateData_1 = { accumulateData: accumulateData$3 };

const { compose: compose$2 } = compose_1;
const { accumulateData: accumulateData$2 } = accumulateData_1;

function parseOptionalArgs(...args) {
  const options = {
    accumulator: [],
    ...(typeof args[args.length - 1] === "object" ? args.pop() : {}),
  };
  const size = options.size || 1;

  return {
    shouldFlush: args.pop() || ((group) => group.length === size),
    options,
  };
}

function groupData$1(...args) {
  const { shouldFlush, options } = parseOptionalArgs(...args);

  return compose$2(
    accumulateData$2((acc, data, flush) => {
      const group = [...acc, data];
      if (shouldFlush(group)) {
        flush(group);
        return [];
      } else {
        return group;
      }
    }, options)
  );
}

var groupData_1 = { groupData: groupData$1 };

const { Transform } = require$$0;

/**
 * Inspired by https://stackoverflow.com/a/43811543/122975
 */
class TransformArray extends Transform {
  constructor(options) {
    super(options);
    this._resumeTransform = null;
  }
  _transform(chunk, encoding, callback) {
    const array = Array.isArray(chunk) ? chunk : [chunk];
    let index = 0;
    const pushArrayItems = () => {
      while (index < array.length) {
        const item = array[index++];
        const backpressure = !this.push(item);
        if (backpressure) {
          //Stop pushing items and prepare process to be resumed when read function will be called again.
          this._resumeTransform = pushArrayItems;
          return;
        }
      }

      this._resumeTransform = null;
      return callback();
    };

    //Start pushing array items
    pushArrayItems();
  }
  _read(size) {
    if (this._resumeTransform !== null) {
      //Ensure every items from the previous transform has been pushed
      this._resumeTransform();
    }
    super._read(size);
  }
}

function flattenArray$2(options = {}) {
  return new TransformArray({ objectMode: true, ...options });
}

var flattenArray_1 = { flattenArray: flattenArray$2 };

const { accumulateData: accumulateData$1 } = accumulateData_1;
const { flattenArray: flattenArray$1 } = flattenArray_1;
const { compose: compose$1 } = compose_1;

function readLineByLine$1() {
  return compose$1(
    accumulateData$1(
      (acc, data, flush) => {
        const lines = data.toString().split(/\r?\n/);
        const rest = lines.pop();

        if (lines.length > 0) {
          lines[0] = acc + lines[0];
          flush(lines);
          return rest;
        }

        return acc + rest;
      },
      { accumulator: "", readableObjectMode: true, writableObjectMode: false }
    ),
    flattenArray$1()
  );
}

var readLineByLine_1 = { readLineByLine: readLineByLine$1 };

const { PassThrough: PassThrough$1 } = require$$0;
const { decorateWithAsyncIterator } = decorateWithAsyncIterator_1;
const { parseArgs: parseArgs$1 } = parseArgs_1;

function mergeStreams$1(...args) {
  const { streams, options } = parseArgs$1(args);
  const last = streams[streams.length - 1];
  const passThrough = new PassThrough$1(options);
  passThrough.setMaxListeners(0);
  let cpt = streams.length;

  streams.forEach((s) => pipeStream(s));

  async function pipeStream(obj) {
    const stream = typeof obj === "function" ? await obj() : obj;
    stream.on("error", (e) => passThrough.emit("error", e));
    stream.once("end", () => {
      if (--cpt === 0) {
        passThrough.end();
      }
    });

    stream.pipe(passThrough, { end: false });
  }

  if (last.readable) {
    decorateWithAsyncIterator(passThrough);
  }

  return passThrough;
}

var mergeStreams_1 = { mergeStreams: mergeStreams$1 };

const { PassThrough } = require$$0;
const { parseArgs } = parseArgs_1;

class ArrayCursor {
  constructor(array) {
    this.array = array;
    this.cpt = 0;
  }

  next() {
    return this.array[this.cpt++];
  }
}

function isFunction(streams) {
  return streams.length === 1 && typeof streams[0] === "function";
}

function concatStreams$1(...args) {
  const { streams, options } = parseArgs(args);
  const cursor = isFunction(streams) ? { next: streams[0] } : new ArrayCursor(streams);
  const passThrough = new PassThrough({ objectMode: true, ...options });
  passThrough.setMaxListeners(0);

  async function concat() {
    const stream = await cursor.next();
    if (!stream) {
      return passThrough.end();
    }

    stream.on("error", (e) => passThrough.emit("error", e));
    stream.on("end", () => concat());
    stream.pipe(passThrough, { end: false });
  }

  concat();

  return passThrough;
}

var concatStreams_1 = { concatStreams: concatStreams$1 };

const { Writable: Writable$1 } = require$$0;
const cyclist = require$$1;
const { inherits } = require$$2;

const ParallelWrite = function (maxParallel, opts, onWrite) {
  if (!(this instanceof ParallelWrite)) return new ParallelWrite(maxParallel, opts, onWrite);

  if (typeof maxParallel === "function") {
    onWrite = maxParallel;
    opts = null;
    maxParallel = 1;
  }
  if (typeof opts === "function") {
    onWrite = opts;
    opts = null;
  }

  if (!opts) opts = {};
  if (!opts.highWaterMark) opts.highWaterMark = Math.max(maxParallel, 16);
  if (opts.objectMode !== false) opts.objectMode = true;

  Writable$1.call(this, opts);

  this._maxParallel = maxParallel;
  this._onWrite = onWrite;
  this._destroyed = false;
  this._flushed = false;
  this._ordered = opts.ordered !== false;
  this._buffer = this._ordered ? cyclist(maxParallel) : [];
  this._top = 0;
  this._bottom = 0;
  this._ondrain = null;
};

inherits(ParallelWrite, Writable$1);

ParallelWrite.prototype._destroy = function () {
  if (this._destroyed) return;
  this._destroyed = true;
  this.emit("close");
};

ParallelWrite.prototype._write = function (chunk, enc, callback) {
  const self = this;
  const pos = this._top++;

  this._onWrite(chunk, function (err, data) {
    if (self._destroyed) return;
    if (err) {
      self.emit("error", err);
      self.destroy();
      return;
    }
    if (self._ordered) {
      self._buffer.put(pos, data === undefined || data === null ? null : data);
    } else {
      self._buffer.push(data);
    }
    self._drain();
  });

  if (this._top - this._bottom < this._maxParallel) return callback();
  this._ondrain = callback;
};

ParallelWrite.prototype._final = function (callback) {
  this._flushed = true;
  this._ondrain = callback;
  this._drain();
};

ParallelWrite.prototype._drain = function () {
  if (this._ordered) {
    while (this._buffer.get(this._bottom) !== undefined) {
      this._buffer.del(this._bottom++);
    }
  } else {
    while (this._buffer.length > 0) {
      this._buffer.pop();
      this._bottom++;
    }
  }

  if (!this._drained() || !this._ondrain) return;

  const ondrain = this._ondrain;
  this._ondrain = null;
  ondrain();
};

ParallelWrite.prototype._drained = function () {
  const diff = this._top - this._bottom;
  return this._flushed ? !diff : diff < this._maxParallel;
};

function writeData$1(handleChunk, options = {}) {
  return ParallelWrite(options.parallel || 1, { objectMode: true }, async (chunk, callback) => {
    try {
      const res = await handleChunk(chunk);
      callback(null, res);
    } catch (e) {
      callback(e);
    }
  });
}

var writeData_1 = { writeData: writeData$1 };

const { Writable } = require$$0;

function writeToStdout$1() {
  return new Writable({
    write(chunk, encoding, callback) {
      //write to stdout as a side effect and honor finish event
      //https://stackoverflow.com/questions/33190458/node-pipe-to-stdout-how-do-i-tell-if-drained
      process.stdout.write(chunk, callback);
    },
  });
}

var writeToStdout_1 = { writeToStdout: writeToStdout$1 };

var oleoduc = oleoducBareRepoN3a88.oleoduc = oleoduc_1.oleoduc;
var compose = oleoducBareRepoN3a88.compose = compose_1.compose;
var transformData = oleoducBareRepoN3a88.transformData = transformData_1.transformData;
var transformIntoJSON = oleoducBareRepoN3a88.transformIntoJSON = transformIntoJSON_1.transformIntoJSON;
var transformIntoCSV = oleoducBareRepoN3a88.transformIntoCSV = transformIntoCSV_1.transformIntoCSV;
var transformIntoStream = oleoducBareRepoN3a88.transformIntoStream = transformIntoStream_1.transformIntoStream;
var filterData = oleoducBareRepoN3a88.filterData = filterData_1.filterData;
var accumulateData = oleoducBareRepoN3a88.accumulateData = accumulateData_1.accumulateData;
var groupData = oleoducBareRepoN3a88.groupData = groupData_1.groupData;
var readLineByLine = oleoducBareRepoN3a88.readLineByLine = readLineByLine_1.readLineByLine;
var flattenArray = oleoducBareRepoN3a88.flattenArray = flattenArray_1.flattenArray;
var mergeStreams = oleoducBareRepoN3a88.mergeStreams = mergeStreams_1.mergeStreams;
var concatStreams = oleoducBareRepoN3a88.concatStreams = concatStreams_1.concatStreams;
var writeData = oleoducBareRepoN3a88.writeData = writeData_1.writeData;
var writeToStdout = oleoducBareRepoN3a88.writeToStdout = writeToStdout_1.writeToStdout;

export { accumulateData, compose, concatStreams, oleoducBareRepoN3a88 as default, filterData, flattenArray, groupData, mergeStreams, oleoduc, readLineByLine, transformData, transformIntoCSV, transformIntoJSON, transformIntoStream, writeData, writeToStdout };
//# sourceMappingURL=bundle.mjs.map
